/*
 * @(#)$Id: UserInterface.java 5321 2012-04-11 17:43:12Z richard $
 * Copyright (c) 2011 Richard J. Povinelli
 */

import textBoard.TextBoard;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;


/**
 * The user interface for the first pass of the centipede game.
 *
 * @version $LastChangedRevision: 5321 $, $LastChangedDate: 2012-04-11 12:43:12 -0500 (Wed, 11 Apr 2012) $
 * @author $LastChangedBy: richard $
 */
public class UserInterface implements KeyListener {

  /** Number of columns for the text board. */
  private static final int COLUMNS = 30;

  /** Key code for the down arrow. */
  private static final int DOWN_ARROW = 40;

  /** Default font size for the text board. */
  private static final int FONT_SIZE = 14;

  /** Key code for the left arrow. */
  private static final int LEFT_ARROW = 37;

  /** Key code for the right arrow. */
  private static final int RIGHT_ARROW = 39;

  /** Number of rows for the text board. */
  private static final int ROWS = 15;

  /** Key code for the up arrow. */
  private static final int UP_ARROW = 38;

  /** The physics ship. */
  private Ship ship;

  /** The text based interface. */
  private TextBoard textBoard;

  /**
   * Constructs the user interface. Places the ship, constructs the text board, and sets up the key listener.
   */
  public UserInterface() {
    ship      = new Ship(5, 5);
    textBoard = new TextBoard(ROWS, COLUMNS, FONT_SIZE);
    textBoard.addKeyListener(this);
  }

  /**
   * Handles key pressed events. Maps the arrow keys to changes in the ships direction.
   *
   * @param e The event that triggered this method.
   */
  @Override
  public void keyPressed(KeyEvent e) {
    switch (e.getKeyCode()) {
      case UP_ARROW:
        ship.setDyPositive();
        break;
      case DOWN_ARROW:
        ship.setDyNegative();
        break;
      case RIGHT_ARROW:
        ship.setDxPositive();
        break;
      case LEFT_ARROW:
        ship.setDxNegative();
        break;
    }
  }

  /**
   * Not used.
   *
   * @param e N/A
   */
  @Override
  public void keyReleased(KeyEvent e) {}

  /**
   * Not used.
   *
   * @param e N/A
   */
  @Override
  public void keyTyped(KeyEvent e) {}

  /**
   * Creates an instance of the user interface and runs it.
   *
   * @param args Not used.
   */
  public static void main(String[] args) {
    UserInterface userInterface = new UserInterface();
    userInterface.run();
  }

  /**
   * The main loop of the program. Gets the text matrix, turns on the text board, and loops through the process of
   * getting the location of the ship and displaying the ship.
   *
   */
  private void run() {
    double   x    = 0;
    double   y    = 0;
    char[][] text = textBoard.getText();
    textBoard.run();

    while (true) {
      textBoard.setText(text, ' ');
      ship.tick();
      x                      = ship.getX();
      y                      = ship.getY();
      text[(int) y][(int) x] = '@';
      textBoard.repaint();

      try {
        Thread.sleep(250);
      }
      catch (InterruptedException e) {
        e.printStackTrace();
      }
    }
  }
}
